﻿﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Content;

namespace SpaceShooter
{
    // =======================================================================
    // GameState, statisk klass med metoder för olika gamestates. Varje game-
    // state har två metoder vardera. En för update-funktionalitet och en för
    // draw-funktionalitet. Quit har inga egna metoder utan gör så att spelet
    // avslutas direkt.
    // =======================================================================
    static class GameElements
    {
        static Menu menu;
        static Player player;
        static Background background;
        static List<Enemy> enemies;
        static List<GoldCoin> goldCoins;
        static Texture2D goldCoinSprite;

        public enum State { Menu, Run, HighScore, Quit }; // olika gamestates
        public static State currentState;

        // =======================================================================
        // Initialize(), anropas av Game1.Initialize() då spelet startar. Här
        // ligger all kod för att  initiera objekt och skapa demdock inte laddning
        // av olika filer (bilder, ljud etc).
        // =======================================================================
        public static void Initialize()
        {
            goldCoins = new List<GoldCoin>();
        }

        // =======================================================================
        // LoadContent(), anropas av Game1.LoadContent() då spelet startar. Här 
        // laddas alla objekt/filer in (bilder, ljud etc)
        // =======================================================================
        public static void LoadContent(ContentManager content, GameWindow window)
        {
            // Skapa menyn:
            menu = new Menu((int)State.Menu);
            menu.AddItem(content.Load<Texture2D>("images/menu/start"), (int)State.Run);
            menu.AddItem(content.Load<Texture2D>("images/menu/highscore"), (int)State.HighScore);
            menu.AddItem(content.Load<Texture2D>("images/menu/exit"), (int)State.Quit);

            // Skapa spelarobjektet:
            player = new Player(content.Load<Texture2D>("images/player/ship"), 380, 400, 2.5f, 4.5f, content.Load<Texture2D>("images/player/bullet"));

            // Skapa bakgrundsbilden:
            background = new Background(content.Load<Texture2D>("images/background"), window);

            // Skapa fiender
            enemies = new List<Enemy>();
            Random random = new Random();
            Texture2D tmpSprite = content.Load<Texture2D>("images/enemies/mine");
            for (int i = 0; i < 5; i++)
            {
                int rndX = random.Next(0, window.ClientBounds.Width - tmpSprite.Width);
                int rndY = random.Next(0, window.ClientBounds.Height / 2);
                Mine temp = new Mine(tmpSprite, rndX, rndY);
                enemies.Add(temp); // Lägg till i listan
            }
            tmpSprite = content.Load<Texture2D>("images/enemies/tripod");
            for (int i = 0; i < 5; i++)
            {
                int rndX = random.Next(0, window.ClientBounds.Width - tmpSprite.Width);
                int rndY = random.Next(0, window.ClientBounds.Height / 2);
                Tripod temp = new Tripod(tmpSprite, rndX, rndY);
                enemies.Add(temp); // Lägg till i listan
            }


            goldCoinSprite = content.Load<Texture2D>("images/powerups/coin");
        }

        // =======================================================================
        // Reset(), återställer alla objekt så att man har möjligheten att starta
        // ett nytt spel. (Denna metod ser mycket ut som LoadContent(). Innehåll-
        // et i de båda bör läggas i en separat metod som de båda anropar...
        // =======================================================================
        private static void Reset(GameWindow window, ContentManager content)
        {
            player.Reset(380, 400, 2.5f, 4.5f);

            // Skapa fiender
            enemies.Clear();
            Random random = new Random();
            Texture2D tmpSprite = content.Load<Texture2D>("images/enemies/mine");
            for (int i = 0; i < 5; i++)
            {
                int rndX = random.Next(0, window.ClientBounds.Width - tmpSprite.Width);
                int rndY = random.Next(0, window.ClientBounds.Height / 2);
                Mine temp = new Mine(tmpSprite, rndX, rndY);
                enemies.Add(temp); // Lägg till i listan
            }
            tmpSprite = content.Load<Texture2D>("images/enemies/tripod");
            for (int i = 0; i < 5; i++)
            {
                int rndX = random.Next(0, window.ClientBounds.Width - tmpSprite.Width);
                int rndY = random.Next(0, window.ClientBounds.Height / 2);
                Tripod temp = new Tripod(tmpSprite, rndX, rndY);
                enemies.Add(temp); // Lägg till i listan
            }
        }

        // =======================================================================
        // MenuUpdate(), kontrollerar om användaren väljer något av menyvalen
        // =======================================================================
        public static State MenuUpdate(GameTime gameTime)
        {
            return (State)menu.Update(gameTime);
        }

        // =======================================================================
        // MenuDraw(), ritar ut menyn
        // =======================================================================
        public static void MenuDraw(SpriteBatch spriteBatch)
        {
            background.Draw(spriteBatch);
            menu.Draw(spriteBatch);
        }

        // =======================================================================
        // RunUpdate(), update-metod för själva spelet.
        // =======================================================================
        public static State RunUpdate(ContentManager content, GameWindow window, GameTime gameTime)
        {
            background.Update(window);
            player.Update(window, gameTime); // Uppdatera spelarens position

            // Gå igenom alla fiender
            foreach (Enemy e in enemies.ToList())
            {
                // Kontrollera om fienden kolliderar med ett skott
                foreach (Bullet b in player.Bullets)
                {
                    if (e.CheckCollision(b)) // Kollision uppstod
                    {
                        e.IsAlive = false; // Döda fienden
                        player.Points++; // Ge spelaren poäng
                    }
                }

                if (e.IsAlive) // Kontrollera om fienden lever
                {
                    // Kontrollera kollision med spelaren:
                    if (e.CheckCollision(player))
                        player.IsAlive = false;
                    e.Update(window); // Flytta på dem
                }
                else // Ta bort fienden för den är död
                    enemies.Remove(e);
            }

            // Guldmynten ska uppstå slumpmässigt, en chans på 200:
            Random random = new Random();
            int newCoin = random.Next(1, 200);
            if (newCoin == 1) // ok, nytt guldmynt ska uppstå
            {
                // Var ska guldmyntet uppstå:
                int rndX = random.Next(0, window.ClientBounds.Width - goldCoinSprite.Width);
                int rndY = random.Next(0, window.ClientBounds.Height - goldCoinSprite.Height);
                // Lägg till myntet i listan:
                goldCoins.Add(new GoldCoin(goldCoinSprite, rndX, rndY, gameTime));
            }

            // Gå igenom hela listan med existernade guldmynt
            foreach (GoldCoin gc in goldCoins.ToList())
            {
                if (gc.IsAlive) // Kontrollera om guldmyntet lever
                {
                    // gc.Update() kollar om guldmyntet har blivit för gammalt
                    // för att få leva vidare:
                    gc.Update(gameTime);

                    // Kontrollera om de kolliderat med spelaren:
                    if (gc.CheckCollision(player))
                    {
                        goldCoins.Remove(gc); // Ta bort myntet vid kollision...
                        player.Points++; // ...och ge spelaren poäng
                    }
                }
                else // Ta bort guldmyntet för det är dött
                    goldCoins.Remove(gc);
            }

            if (!player.IsAlive) // Spelaren är död
            {
                Reset(window, content); // Återställ alla spelobjekt
                return State.Menu; // återgå till menyn
            }


            return State.Run; // Stanna kvar i Run

        }

        // =======================================================================
        // RunDraw(), metod för att rita ut "själva spelet"
        // =======================================================================
        public static void RunDraw(SpriteBatch spriteBatch)
        {
            background.Draw(spriteBatch);
            player.Draw(spriteBatch);
            foreach (Enemy e in enemies)
                e.Draw(spriteBatch);
            foreach (GoldCoin gc in goldCoins)
                gc.Draw(spriteBatch);
        }

        // =======================================================================
        // HighScoreUpdate(), update-metod för highscore-listan
        // =======================================================================
        public static State HighScoreUpdate()
        {
            KeyboardState keyboardState = Keyboard.GetState();
            // Återgå till menyn om man trycker ESC:
            if (keyboardState.IsKeyDown(Keys.Escape))
                return State.Menu;
            return State.HighScore; // Stanna kvar i HighScore

        }

        // =======================================================================
        // HighScoreDraw(), ritar ut highscorelistan
        // =======================================================================
        public static void HighScoreDraw(SpriteBatch spriteBatch)
        {
            // Rita ut highscore-listan:
        }
    }
}